#ifndef VHTTPD_MESSAGE_H
#define VHTTPD_MESSAGE_H

#include <stdarg.h>
#include <sys/types.h>

#include <vhttpd/codes.h>

typedef struct _message_t message_t;

message_t *message_new (void);
void message_delete (message_t *);

message_t *message_request_new (void);
message_t *message_response_new (retcode_t);

int message_get_plain (const message_t *);
void message_set_plain (message_t *, int);

void message_set_header_vprintf (message_t *, const char *, const char *, va_list)
  __attribute__ ((__format__ (__printf__, 3, 0)));
void message_set_header_printf (message_t *, const char *, const char *, ...)
  __attribute__ ((__format__ (__printf__, 3, 4)));

void message_set_header (message_t *, const char *, const char *);
const char *message_get_header (const message_t *, const char *);

void message_set_cookie_vprintf (message_t *, const char *, const char *, va_list)
  __attribute__ ((__format__ (__printf__, 3, 0)));
void message_set_cookie_printf (message_t *, const char *, const char *, ...)
  __attribute__ ((__format__ (__printf__, 3, 4)));

void message_set_cookie (message_t *, const char *, const char *);
const char *message_get_cookie (const message_t *, const char *);

void message_set_body (message_t *, const char *, size_t);
const char *message_get_body (const message_t *);

void message_set_startline (message_t *, const char *);
const char *message_get_startline (const message_t *);

void message_set_file (message_t *, int, size_t);
int message_get_file (const message_t *);

int message_get_length (const message_t *);
const char *message_get_method (const message_t *);
const char *message_get_uri (const message_t *);
const char *message_get_query (const message_t *);
retcode_t message_get_code (const message_t *);

char *message_header_field (char *, char **);
#endif
